@@ -3,32 +3,33 @@ import os
 
 def which(program):
     """
-    Returns whether the given program is executable.
+    Returns the file path of the first executable of the given program
+    name in the system ``PATH`` environment variable.
+    
     This function is a system-independent Python equivalent of the \*nix
     ``which`` command.
     
-    The implementation is stolen from
-    http://stackoverflow.com/questions/377017/test-if-executable-exists-in-python.
+    :Note: The implementation is adapted from
+        http://stackoverflow.com/questions/377017/test-if-executable-exists-in-python.
     
-    :param program: the program to check
-    :return: whether the given program is an exectuable file
-        in the system path
+    :param program: the program name to check
+    :return: the filename in the system path
     """
     fpath, fname = os.path.split(program)
     if fpath:
-        if is_executable(program):
+        if _is_executable(program):
             return program
     else:
         for path in os.environ["PATH"].split(os.pathsep):
             path = path.strip('"')
             exe_file = os.path.join(path, program)
-            if is_executable(exe_file):
+            if _is_executable(exe_file):
                 return exe_file
 
     return None
 
 
-def is_executable(fpath):
+def _is_executable(fpath):
     """
     :param fpath: the file path to check
     :return: whether the file exists and is executable
